/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       gclamp.h                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       31-Dec-2010 17.59.41 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header file                                                               *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/
#ifndef __GCLAMP_H__
#define __GCLAMP_H__

#include <list>
#include <string>

#include <string.h>
#include <gtk/gtk.h>

#include "logfile.h"
#include "requester.h"
#include "view.h"

using namespace std;

enum action{LEAVE, MOVE, COPY, DELETE};



class Gclamp
{
public:
  Gclamp(int argc, char *argv[]);
  ~Gclamp(void);

  bool isdir(string pathname);
  bool isfile(string pathname, off_t *file_size);

  string dtos(double n);
  string itos(int n);

  bool isseparator(char x);
  void getright(string &s, string &v);
  bool getvalue(string &line, string &name, string &value);
  bool getvalue(fstream &ifp,
                string &line,
                string &startname,
                string &endname,
                list<string> &value);

  void remove_leading(string &s);
  void remove_trailing(string &s);
  bool getline(fstream &ifp, string &line, bool clear_leadin);
  string file_contents_to_string(string pathname);

  string home_directory;
  string quarantine_directory;

  time_t start_time;

  bool want_log;
  bool append_log;
  bool log_empty_files;
  bool log_OK_files;
  bool log_excluded_files;

  class View *view_gclamp_log;
  class View *view_freshclam_log;
  class View *view_gclamp_defaults;
  class View *view_about;

  gint width;
  gint height;


  static GdkPixbuf *icon_pixbuf;
  bool quit_flag;

private:
  gboolean program_running(char *command);
  bool is_running(string command, int &pid, Gclamp *g);
//  static void callback_visible(GtkWidget *widget, Gclamp *g);
  static void callback_visible(Gclamp *g);
//  static gboolean callback_visible(Gclamp *g);
  static void callback_quit(GtkWidget *widget, gpointer data);
  
  static void callback_browse(GtkWidget *widget, Gclamp *g);
  static void callback_about(GtkWidget *widget, Gclamp *g);

  static void callback_scan(GtkWidget *widget, Gclamp *g);
  static void callback_scan_all(GtkWidget *widget, Gclamp *g);
  static void callback_update_db(GtkWidget *widget, Gclamp *g);

  static gboolean check_database(gpointer gdata);
  static void callback_restart_as_root(GtkWidget *widget, Gclamp *g);
  static void callback_view_freshclam_log(GtkWidget *widget, Gclamp *g);
  static void callback_view_gclamp_log(GtkWidget *widget, Gclamp *g);
  static void callback_view_gclamp_defaults(GtkWidget *widget, Gclamp *g);

  static void callback_save_defaults(GtkWidget *widget, Gclamp *g);
  static void callback_load_defaults(GtkWidget *widget, Gclamp *g);

  static void callback_log_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_log_group_checkbutton_clicked(GtkWidget *widget, Gclamp *g);

  static void callback_move_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_copy_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_delete_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_ignore_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_su_checkbutton_clicked(GtkWidget *widget, Gclamp *g);
  static void callback_auto_update_checkbutton_clicked(GtkWidget *widget, Gclamp *g);

  void decorate_button(GtkWidget *widget, GdkColor *bg_normal,  GdkColor *bg_prelight, const gchar *stock_id);

  void fix_db_status(bool want_summary, Gclamp *g);
  int scan_process(string command, Gclamp *g);

  string get_file(string command);
  void get_info_from_widgets(Gclamp *g);
    
  static int do_scan(void *gclamp);
  static int do_update(void *gclamp);
  
  bool scan_all;
  static bool do_scan_all;

  bool have_su_command(string &msg, Gclamp *g);
    
  string get_db_date(Gclamp *g);
  int get_db_days(Gclamp *g);

  static gboolean busy_world(gpointer gdata);
  void start_timer(Gclamp *g);
  void stop_timer(Gclamp *g);

  void scan_file(string pathname, Gclamp *g);
  void scan_directory(Directory directory, Gclamp *g);
  void clear_info(Gclamp *g);

  GtkWidget *add_page(GtkWidget *notebook, gchar *title);
  void fill_action_page(GtkWidget *vbox);
  void fill_directories_page(GtkWidget *vbox);
  void fill_directories_omit_page(GtkWidget *vbox);
  void fill_files_page(GtkWidget *vbox);
  void fill_files_omit_page(GtkWidget *vbox);
  void fill_options_page(GtkWidget *vbox);

  GtkWidget *create_bar_submenu (GtkWidget *menubar, char *name);
  GtkWidget *create_menuitem (GtkWidget *window,
                            GtkWidget *menu, 
				                    char *name,
                            GtkAccelGroup *accel_group,
				                    char *accel,
                            GtkTooltips *tooltips,
				                    char *tip, 
				                    GtkSignalFunc func,
				                    gpointer data);
  GtkWidget *create_spinner(int s_start, int s_min, int s_max);

  GtkWidget *window;
  GtkTooltips *tooltips;

  GtkWidget *scanned_label;
  GtkWidget *infected_label;
  GtkWidget *display;

  string update_message;

  string known_viruses;
  string engine_version;

  int num_scanned;
  int num_infected;
  int num_denied;
  int num_read_errors;
  int num_open_errors;
  int num_empty;
  int num_excluded;

  double data_scanned;
  double data_read;

  gint warn_days;
  gint update_days;

  static GtkWidget *warning_label;
  static GtkWidget *status_label;
  static GtkWidget *db_status_label;
  static GtkWidget *summary_label;
  static string summary;

  GtkWidget *current_file_label;
  string current_file;


  list<Directory> directories;
  list<string> directories_omit;
  list<string> files;
  list<string> files_omit;

  bool busy;

  GtkAccelGroup *accel_group;

  Requester *frd;
  Requester *frd_omit;
  Requester *frf;
  Requester *frf_omit;

  class Logfile *logfile;

  action infected_action;
  string browser_command;
  string su_command;       // possibles are:- -e0 -e1 or -e2 (su+sudo : su : sudo)
//  int su_timeout;

  GtkWidget *log_checkbutton;
  GtkWidget *log_append_checkbutton;
  GtkWidget *log_overwrite_checkbutton;
  GtkWidget *log_include_empty_checkbutton;
  GtkWidget *log_include_OK_checkbutton;
  GtkWidget *log_include_excluded_checkbutton;

  GtkWidget *move_checkbutton;
  GtkWidget *copy_checkbutton;
  GtkWidget *delete_checkbutton;
  GtkWidget *ignore_checkbutton;

  GtkWidget *browser_entry;

//  GtkWidget *xtermsu_checkbutton;
//  GtkWidget *kdesu_checkbutton;
//  GtkWidget *gksu_checkbutton;
  GtkWidget *susudo_checkbutton;
  GtkWidget *su_checkbutton;
  GtkWidget *sudo_checkbutton;


//  GtkWidget *su_timeout_spinner;
  
  GtkWidget *auto_update_checkbutton;
  GtkWidget *db_warn_spinner;
  GtkWidget *auto_update_spinner;

  static string Want_Log_TXT;
  static string Append_Log_TXT;
  static string Log_Empty_Files_TXT;
  static string Log_OK_Files_TXT;
  static string Log_Excluded_Files_TXT;
  static string Browser_Command_TXT;
  static string Use_Su_TXT;
  static string Su_Timeout_TXT;
  static string Action_TXT;
  static string Warn_Days_TXT;
  static string Auto_Update_Days_TXT;
  static string Start_Directories_TXT;
  static string End_Directories_TXT;
  static string Start_Directories_Omit_TXT;
  static string End_Directories_Omit_TXT;
  static string Start_Files_TXT;
  static string End_Files_TXT;
  static string Start_Files_Omit_TXT;
  static string End_Files_Omit_TXT;

  static string month_names[];

  static string freshclam_log_warning;
  static string freshclam_log_fname;
  
  static const guint8 load_pixbuf_src[];
  static GtkWidget *load_image;

  static const guint8 save_pixbuf_src[];
  static GtkWidget *save_image;

  static const guint8 icon_pixbuf_src[];

  static const guint8 rw0_pixbuf_src[];
  static const guint8 rw1_pixbuf_src[];
  static const guint8 rw2_pixbuf_src[];
  static const guint8 rw3_pixbuf_src[];
  static const guint8 rw4_pixbuf_src[];
  static const guint8 rw5_pixbuf_src[];
  static const guint8 rw6_pixbuf_src[];

  static GdkPixbuf *rw_pixbuf[];

  static const guint8 blank_pixbuf_src[];
  static GdkPixbuf *blank_pixbuf;

/*
  static GdkColor red;
  static GdkColor white;
  static GdkColor blue;
  static GdkColor black;
  static GdkColor pink;
  static GdkColor lightblue;
  static GdkColor lightgreen;
  static GdkColor lightbrown;
  static GdkColor lightyellow;
*/

  string gclamp_doc_pathname;
  string gclamp_pathname;
  string update_pathname;
  
  guint timer;
  guint db_timer;

  GtkWidget *gclamp_image;
//  GtkWidget *busy_image;
  guint world_num;
  static guint visible_signal;

protected:

};

#endif // __GCLAMP_H__
