/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       logfile.cpp                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       19-May-2009 12.16.14 -36000 0                             *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/


#include <string.h>
#include <sys/stat.h>
#include "logfile.h"

#include "RH_version.h"

using namespace std;

fstream Logfile::logfs;

Logfile::Logfile(Gclamp *g)
{
//cout << "Logfile:" << endl;

logname = g->home_directory + "/gclamp.log";
this->g = g;
}
        
Logfile::~Logfile(void)
{
this->close();
//cout << "~Logfile:" << endl;
}
         



/****************************************
 * Open the log file if requested.      *
 * Put it in same directory.            *
 ****************************************/
 
// If first we open a temporary log file and use that
// Later in Getopts::ReadEnv this may become the actual file
// or be appended to the pre-existing log file.
// The tempoaray file is then eliminated.
// If no log is wanted, both the temorary and the original are removed.


void Logfile::open(void)
{
if (g->want_log)
  {
  string buffer;
  string *t;
  int len;

  if ( ! (logfs.is_open()) )
    {
    string s = RH_VERSION_STRING;
    buffer = "\n" + s;
    s.clear();
    for (len=buffer.length()-1; len>0; len--)
      {
      s += '-';
      }
    buffer = s + buffer + "\n" + s;

    if (g->append_log) logfs.open( (char *)logname.c_str(), ios::out | ios::app | ios::binary );     
    else logfs.open( (char *)logname.c_str(), ios::out | ios::trunc | ios::binary );

    if (logfs.is_open())
      {
      write2log( buffer );
                      
      buffer = "OPENED " + logname + " : " + asctime(localtime(&g->start_time));

      write2log( buffer );
      chmod(logname.c_str(), 0677); // make sure it is read/write by everyone
      }
    else
      {
      // ERROR
      }
    }
  }
return;
}


void Logfile::close(void)
{
if (logfs.is_open()) logfs.close();
}

// Write ane entry in log file
void Logfile::write2log(string &s)
{
string buffer;

//cout << "Logfile::write2log 1"<< endl;

buffer += s + "\n";

// This shouldn't be necessary
if ( ! (logfs.is_open()) )
  {
  open();  // If we want a log file it gets opened if not already opened and we can use it
  }

if ( logfs.is_open() )
  {    
  if ( ! (logfs.write( (char *)buffer.c_str(), buffer.length() ) ) )
    {
//    cout << "Logfile::write2log 4" << endl;
      
    // ERROR
//    cerr << "Logfile::write2log - FAILED to write " << buffer << "\n   : errno=" << errno << " : " << strerror(errno) << endl;
    }
    
  logfs.flush();
  }
return;
}


