/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       requester.cpp                                             *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Apr-2010 17.23.05 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* File requester module                                                     *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <iostream>

#include <stdlib.h>
#include <stdio.h> // stdout
#include <gtk/gtk.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>

#include "requester.h"

using namespace std;

GdkColor *Requester::bg_normal = NULL;
GdkColor *Requester::bg_prelight = NULL;
const gchar *Requester::cancel_stock_id = NULL;
const gchar *Requester::ok_stock_id = NULL;

Requester::Requester(void)
{
//cout << "Requester:" << endl;
requester_window = NULL;
directory_pixbuf = NULL;
file_pixbuf = NULL;
exec_pixbuf = NULL;
reply_flag = FALSE;
name_editable = FALSE;
  
omit_recursive = FALSE;

ok_text = "OK";
cancel_text = "CANCEL";
 
name_entry = NULL;
treeview = NULL;


attr = pango_attr_family_new("monospace");
attr_list = pango_attr_list_new();
pango_attr_list_insert(attr_list, attr);
}


Requester::~Requester(void)
{
//cout << "~Requester:" << endl;

if (directory_pixbuf) g_object_unref(directory_pixbuf);
if (file_pixbuf) g_object_unref(file_pixbuf);
if (exec_pixbuf) g_object_unref(exec_pixbuf);
pango_attr_list_unref(attr_list);
}

void Requester::set_name_editable(gboolean editable)
{
name_editable = editable;
}

void Requester::set_recursive_not_included(void)
{
omit_recursive = 1;
}

void Requester::set_title1(string text)
{

title1 = text;
}

void Requester::set_title2(string text)
{
title2 = text;
}

void Requester::set_ok_text(string text)
{
ok_text = text;
}

void Requester::set_cancel_text(string text)
{
cancel_text = text;
}

void Requester::set_decorate_buttons(GdkColor *bg_normal,  GdkColor *bg_prelight,
                                  const gchar *cancel_stock_id, const gchar *ok_stock_id)
{  
this->bg_normal       = bg_normal;
this->bg_prelight     = bg_prelight;
this->cancel_stock_id = cancel_stock_id;
this->ok_stock_id     = ok_stock_id;
}

void Requester::callback_quit_requester(GtkWidget *widget, Requester *fr)
{
gtk_widget_destroy(GTK_WIDGET(fr->requester_window));
}

bool Requester::get_name_from_pathname(gchar *pathname, string &name)
{
if ((strlen(pathname) > 1))
  {
  gchar *p = &pathname[strlen(pathname) - 1];
  while (p > pathname)
    {
    if (*p == '/')
      {
      break;
      }
    p--;
    }
  if ((*p) == '/')
    {
    name = (p+1);
    return TRUE;
    }
  }
name.clear();
return FALSE;
}

void Requester::callback_OK(GtkWidget *widget, Requester *fr)
{
//cout << "Requester::callback_OK -1" << endl;

if (fr->get_mode() == GTK_SELECTION_SINGLE)
  {
//cout << "Requester::callback_OK -2" << endl;

  fr->chosen_name = gtk_entry_get_text(GTK_ENTRY(fr->name_entry));

  if ( (!fr->chosen_name.empty() ) )
    {
    fr->reply_flag = TRUE;
    }
  else fr->reply_flag = FALSE;

//cout << "Requester::callback_OK -3" << endl;

  if (fr->get_type()==reqtype(REQ_FILES))
    {
//cout << "Requester::callback_OK -4" << endl;

    fr->chosen_f_pathname = gtk_entry_get_text(GTK_ENTRY(fr->path_entry));
    fr->chosen_f_pathname += "/" + fr->chosen_name;

//cout << "Requester::callback_OK -5 GOT pathname " << fr->chosen_f_pathname << endl;
//cout << "Requester::callback_OK -5 GOT     name " << fr->chosen_name << endl;

    }
  else
    {
    string name;
    fr->chosen_name = gtk_entry_get_text(GTK_ENTRY(fr->name_entry));
    name = gtk_entry_get_text(GTK_ENTRY(fr->path_entry));
    name += "/" + fr->chosen_name;
    fr->chosen_d_pathname.name = name;
    fr->chosen_d_pathname.recursive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fr->recursive_checkbutton));
    }
  }

else // MULTI
  {
  // If we have anything in treeview2 return fr->reply_flag = TRUE;
  // Check if the first iter is valid...
  gchar *str;
  string name;
  GtkTreeIter iter;
  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(fr->treeview2));
  
  fr->chosen_f_pathname_list.clear();
  fr->chosen_d_pathname_list.clear();
  
  fr->chosen_name_list.clear();

//cout << "Requester::callback_quit_requeste:-" << endl;

  gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
  if (valid) fr->reply_flag = TRUE;

  while (valid)
    {
    if (fr->display_type==reqtype(REQ_FILES))
      {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
                         1,
                         &str,
                         -1);
      fr->chosen_f_pathname_list.push_back(str);
      if (fr->get_name_from_pathname(str, name))
        {
        fr->chosen_name_list.push_back(name);
        }    
      }
    else
      {
      gchar *rec;
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                     1, &rec,
                     2, &str,
                    -1);

      Directory d;
      d.name = str;
      d.recursive = (strcmp(rec, "R")) ? TRUE : FALSE;
      
      fr->chosen_d_pathname_list.push_back( d );
            
      if (fr->get_name_from_pathname(str, name))
        {
        fr->chosen_name_list.push_back(name);
        }    
      
      g_free(rec);
      }
      
      g_free(str);
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
      }
  }

callback_quit_requester(widget, fr);
}

void Requester::callback_destroy_requester(GtkWidget *widget, gpointer data)
{
gtk_grab_remove(GTK_WIDGET(widget));
gtk_main_quit();
}

bool Requester::in_list(list<string> slist, string s)
{
list<string>::iterator i;

for (i=slist.begin(); i!=slist.end(); ++i)
  {
  if ( (*i)==s ) return TRUE;
  }
return FALSE;
}



gboolean Requester::isfile(const char *pathname)
{
struct stat64 statbuf;

if (!(lstat64(pathname, &statbuf)))
  {
  if (S_ISREG(statbuf.st_mode)) return(TRUE);
  }

return(FALSE); // Not a regular file or ERROR
}


gboolean Requester::isdir(const char *pathname)
{
struct stat64 statbuf;
if (!(lstat64(pathname, &statbuf)))
  {
  if (S_ISDIR(statbuf.st_mode)) return(TRUE);
  }
return(FALSE);
}

gboolean Requester::get_name_lists(string directory, list<string> &dirlist, list<string> &filelist)
{
DIR *dp;
struct dirent *de;
string fname;
string pathname;

dirlist.clear();
filelist.clear();
  
if ( (dp = opendir((char *)directory.c_str())) )
    {
    while ( (de = readdir(dp)) )
        {
        fname = de->d_name;
 
        pathname = directory + "/" + fname;

        if (isfile(pathname.c_str()))
          {
          filelist.push_back(fname);
          filelist.sort();
          filelist.unique();
          }
        else if (isdir(pathname.c_str()))
          {
          dirlist.push_back(fname);
          dirlist.sort();
          dirlist.unique();
          }
        }
    closedir(dp);
    return TRUE;
    }
return FALSE;
}
  
void Requester::clear_window2(Requester *fr)
{
GtkTreeModel *model;
GtkTreeIter iter;

if ( (model = gtk_tree_view_get_model(GTK_TREE_VIEW(fr->treeview2))) )
	{
  gtk_list_store_clear(GTK_LIST_STORE(model));
  }
}


// Should make this so that rec is a   parameter/argument
void Requester::append_treeview2(gchar *str, bool recursive, Requester *fr)
{
GtkTreeIter iter;
GtkTreeIter sibling;
GdkPixbuf *pixbuf;
gchar *test_str;
string s;
gboolean valid;

if (fr->display_type == reqtype(REQ_FILES))
  {
  if (access(str, X_OK)==0) pixbuf = fr->exec_pixbuf;
  else pixbuf = fr->file_pixbuf;
  }
else
  {
  pixbuf = fr->directory_pixbuf;
  }

GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview2));
  
valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

while (valid)
  {
  gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    (fr->display_type == reqtype(REQ_FILES)) ? 1 : 2, // the column #,
                    &test_str,
                    -1);
  s = test_str;
  g_free(test_str);
     
  if (strcmp( str, s.c_str() ) == 0) return;

  if (strcmp( str, s.c_str() ) < 0) // to ignore case to find where to put it use strcasecmp
    {      
    // INSERT IT HERE
    gtk_list_store_insert_before(GTK_LIST_STORE(model),
                                &sibling,
                                &iter);      
    if (fr->display_type == reqtype(REQ_FILES))
      {
      gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, pixbuf, 1, str, 2, "F", -1);
      }
    else
      {
      gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0, pixbuf, 1, (recursive && (!omit_recursive)) ? "R" : "", 2, str, 3, "D", -1);
      }
    break;
    }

  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
  }  // END of while (valid)

// If it hasn't been added then !valid so this gets executed
if (!valid)
  {    
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);

  if (fr->display_type == reqtype(REQ_FILES))
    {
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, pixbuf, 1, str, 2, "F", -1);
    }
  else
    {
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, pixbuf, 1, (recursive && (!omit_recursive)) ? "R" : "", 2, str, 3, "D", -1);
    }
  }
}


void Requester::callback_select(GtkWidget *widget, Requester *fr)
{
gchar *path = (gchar *)gtk_entry_get_text(GTK_ENTRY(fr->path_entry));
//cout << "Requester::callback_select - PATH= " << path << endl;

if (fr->get_type() == reqtype(REQ_DIRECTORIES))
  {
  gchar *str;

  // Get the string from path and put it into the 2nd window
  str = (gchar *)gtk_entry_get_text(GTK_ENTRY(fr->path_entry));

//cout << "Requester::callback_select got " << str << endl;

  fr->append_treeview2(str, TRUE, fr);


  }

else
  {
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *str;
  gchar *type;

  // We need the pathname of the selected file
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(fr->treeview));
  if ( gtk_tree_selection_get_selected(selection, &model, &iter) )
    {
	  gtk_tree_model_get(model, &iter, 
                          1, &str,         // column, location - name
                          2, &type,        //                  - type
                          -1);
    if (strcmp(type, "F")==0)
      {
      string pathname = path;
      if (path[strlen(path)-1] != '/') pathname += "/";
      pathname += str;

//cout << "Requester::callback_select - got FILE " << str << endl;

//cout << "Requester::callback_select - pathname= " << pathname << endl;

      fr->append_treeview2((gchar *)pathname.c_str(), FALSE, fr);
      }
//    else cout << "Requester::callback_select <<-NOT got FILE->>" << endl;


    g_free(str);
    g_free(type);
    }
  }
}

// For both of these we need to locate the item that has been selected if any

void Requester::callback_toggle_recursive(GtkWidget *widget, Requester *fr)
{
//cout << "Requester::callback_toggle_recursive" << endl;

GtkTreeIter iter;
GtkTreeModel *model;

GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(fr->treeview2));
    
if ( gtk_tree_selection_get_selected(selection, &model, &iter) )
  {
  gchar *str;
  gchar *rec;

	gtk_tree_model_get(model, &iter,
                          1, &rec,
                          2, &str,         // column, location - name
                          -1);

  // Remove name from the display
  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);     

  fr->append_treeview2(str, (strcmp(rec, "R")==0) ? FALSE: TRUE, fr);

  g_free(str);
  g_free(rec);
  }
}


void Requester::callback_delete(GtkWidget *widget, Requester *fr)
{
//cout << "Requester::callback_delete" << endl;

GtkTreeIter iter;
GtkTreeModel *model;

GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(fr->treeview2));
    
if ( gtk_tree_selection_get_selected(selection, &model, &iter) )
  {
  gchar *str;

  gtk_tree_model_get(model, &iter, 
                              1, &str,         // column, location
                              -1);

//  cout << "Requester::callback_delete - pathname= " << str << endl;
  g_free(str);
      
  // Remove name from the display
  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);     
     
  }
}

/****************************************************************************************/
// Put selected entry into the 2nd window if multiple
//===================================================
void Requester::callback_list_select(GtkWidget *widget, Requester *fr)
{
GtkTreeModel *model;
GtkTreeIter iter;
GList *list = NULL, *node;
gchar *str;
gchar *type;

  
GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(fr->treeview));
  
//fr->chosen_f_pathname.clear();
//fr->chosen_name.clear();

if (fr->display_mode == GTK_SELECTION_SINGLE)
  {
  if ( gtk_tree_selection_get_selected(selection,
                                   &model,
                                   &iter) )

    {
	  gtk_tree_model_get(model, &iter, 
                          1, &str,         // column, location
                          2, &type,
                          -1);
      
    bool want = TRUE;
    string name = str;
    string n_type = type;
    g_free(str);
    g_free(type);

    string pathname = fr->path;
    if (fr->path[fr->path.length()-1] != '/') pathname += '/';
    pathname += name;
      
    gtk_entry_set_text(GTK_ENTRY(fr->name_entry), "");
      
    if (fr->display_type == reqtype(REQ_FILES))
      {
      if (fr->isdir(pathname.c_str()))
        {
        want = FALSE;
        }
      }
    else if (fr->display_type == reqtype(REQ_DIRECTORIES))
      {
      if ((name == ".") ||  (name == "..")) want = FALSE;
      }
      
    if (want)
      {
//      fr->chosen_pathname = pathname;
//      fr->chosen_name = name;
      
      gtk_entry_set_text(GTK_ENTRY(fr->name_entry), name.c_str());
      }
    }
  }

else if (fr->display_mode == GTK_SELECTION_MULTIPLE)
  {
  list = gtk_tree_selection_get_selected_rows(selection, &model);

  for (node=list; node; node=node->next)
	  {
  	if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)node->data))
  		{
  		gtk_tree_model_get(model, &iter, 
                            1, &str,         // column, location
                            2, &type,
                            -1);

      bool want = TRUE;
      string pathname = fr->path;
      if (fr->path[fr->path.length()-1] != '/') pathname += '/';
      pathname += str;

      if (fr->display_type == reqtype(REQ_FILES))
        {
        if (fr->isdir(pathname.c_str())) want = FALSE;
        }
      else if (fr->display_type == reqtype(REQ_DIRECTORIES))
        {
        if ((strcmp(str, ".")==0) || (strcmp(str, "..")==0)) want = FALSE;
        }

      if (want && (!fr->in_list(fr->chosen_name_list, str)) )
        {
        gtk_tree_selection_unselect_iter(selection, &iter);  
        }
  		}

    
    
    }

  for (node=list; node;  node=node->next)
    {
    gtk_tree_path_free( (GtkTreePath*)(node->data) );
    }
  g_list_free (list);

  }
}


// Should come here with a double click ONLY (row-activated signal)
void Requester::callback_double_click(GtkTreeView *view,
                                      GtkTreePath *path,
                                      GtkTreeViewColumn *column,
                                      Requester *fr)
{
gchar *str;
gchar *type;
GtkTreeIter iter;
GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));

gtk_tree_model_get_iter(model, &iter, path);
    
gtk_tree_model_get(model, &iter, 
                          1, &str,         // column, location
                          2, &type,
                          -1);

string name = str;
string ntype = type;

g_free(str);
g_free(type);

if (ntype == "D")
  {
  // It is a directory, so move to it
  if (name  == "..")
    {
    // Move to parent
            
    unsigned int length = fr->path.length();
    unsigned int pos = length - 1;
    while (pos > 1)
      {
      if (fr->path[pos] == '/')
        {
        fr->path = fr->path.substr(0, pos);
        break;
        }
      pos--;
      }
    if (pos == 1)
      {
      fr->path = "/";
      }            
    }
  else if ((name == ".") || (name == "-"))
    {
    // Refresh this directory
    // path unchanged
    }
  else
    {
    // Move to a child directory if readable
    string childpath = fr->path;
    if (fr->path[fr->path.length()-1] != '/') childpath += '/';
    childpath += name;
    if (access(childpath.c_str(), R_OK) == 0)
      {
      fr->path = childpath;
      }
     }

  // Get the names in this directory
  fr->get_name_lists(fr->path, fr->dir_list, fr->file_list);

  // Now fix up the display
  // Remove all rows from list
  fr->fix_list_store(GTK_LIST_STORE(model), fr);

  if (fr->display_mode == GTK_SELECTION_MULTIPLE)
    {

    }
  else gtk_entry_set_text(GTK_ENTRY(fr->name_entry), "");
         
  gtk_entry_set_text(GTK_ENTRY(fr->path_entry), fr->path.c_str());
  }
return;
}


void Requester::fix_list_store(GtkListStore *store, Requester *fr)
{
GtkTreeIter iter;
list<string>::iterator i;
  
gtk_list_store_clear(GTK_LIST_STORE(store));

if (!fr->dir_list.empty()) // Always have directories I think
  {
  for (i=fr->dir_list.begin(); i!=fr->dir_list.end(); ++i)
    {
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, fr->directory_pixbuf, 1, (gchar *)i->c_str(), 2, "D", -1);
    }
  }
if ( (fr->display_type == reqtype(REQ_FILES)) && (!fr->file_list.empty()) )
  {
  for (i=fr->file_list.begin(); i!=fr->file_list.end(); ++i)
    {
    GdkPixbuf *pixbuf; 
    string name = fr->path;
      
    if (fr->path[fr->path.length()-1] != '/') name += '/';
    name += (*i); 

    if (access(name.c_str(), X_OK)==0) pixbuf = fr->exec_pixbuf;
    else pixbuf = fr->file_pixbuf;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, pixbuf, 1, (gchar *)i->c_str(), 2, "F", -1);
    }
  }

}

GtkTreeModel *Requester::create_model(void)
{
GtkListStore *store;

store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);

fix_list_store(store, this);
    
return GTK_TREE_MODEL(store);
}

void Requester::add_columns (GtkTreeView *treeview, bool extra)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;
int n = 0;

renderer = gtk_cell_renderer_pixbuf_new();
column = gtk_tree_view_column_new_with_attributes("",
                 renderer,
                 "pixbuf", n++,
                 NULL);
//gtk_tree_view_column_set_sort_column_id (column, n++);
gtk_tree_view_append_column(treeview, column);
  
renderer = gtk_cell_renderer_text_new ();  // either File pathname  or. if Directory. "R" or "".
column = gtk_tree_view_column_new_with_attributes("",
                 renderer,
                 "text", n++,
                 NULL);
//gtk_tree_view_column_set_sort_column_id (column, n++);
gtk_tree_view_append_column(treeview, column);

if (extra) // This will be window 2 of directories
  {
//  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes("",
                 renderer,
                 "text", n++,
                 NULL);
//  gtk_tree_view_column_set_sort_column_id (column, n++);
  gtk_tree_view_append_column(treeview, column);
  }
  
renderer = gtk_cell_renderer_text_new ();
column = gtk_tree_view_column_new_with_attributes("",
                 renderer,
                 "text",n++,
                 NULL);
//gtk_tree_view_column_set_sort_column_id (column, n++);
gtk_tree_view_column_set_visible(column, FALSE);        // F or D
gtk_tree_view_append_column(treeview, column);

//cout << "Requester::add_columns  2 n= " << n << endl;
}


GtkWidget *Requester::file_window(GtkWidget *parent, reqtype name_type, GtkSelectionMode mode)
{
GtkTreeModel *model;
GtkWidget *widget;

display_mode = mode;
display_type   = name_type;  // REQ_FILES or REQ_DIRECTORIES

GtkWidget *vbox1 = gtk_vbox_new (FALSE, 0);
GtkWidget *hbox = gtk_hbox_new (TRUE, 0);


GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);  
gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);

if (path.empty())
  {
  // use current directory
  char * cpath = get_current_dir_name();
  path = cpath;
  free(cpath);
  }

get_name_lists(path, dir_list, file_list);

directory_pixbuf = gtk_widget_render_icon(GTK_WIDGET(parent),
                                                     GTK_STOCK_DIRECTORY,
                                                     GTK_ICON_SIZE_MENU,
                                                     NULL);
file_pixbuf = gtk_widget_render_icon(GTK_WIDGET(parent),
                                                GTK_STOCK_FILE,
                                                GTK_ICON_SIZE_MENU,
                                                NULL);
exec_pixbuf = gtk_widget_render_icon(GTK_WIDGET(parent),
                                                GTK_STOCK_EXECUTE,
                                                GTK_ICON_SIZE_MENU,
                                                NULL);

// NOTE: (GtkIconSize)-1 gives an icon that hasn't been re-sized

model = create_model();

treeview = gtk_tree_view_new_with_model(model);
gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);

if (requester_window)
  {
  g_signal_connect(G_OBJECT ( gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)) ),
								 "changed",
								 G_CALLBACK(callback_list_select), this);
  }


g_signal_connect(treeview, "row-activated",     // For double click only
  		   GTK_SIGNAL_FUNC(callback_double_click), this);


// Always use single even in multiple mode, (use 2nd window for multiple) 
gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), GTK_SELECTION_SINGLE);
                            
g_object_unref (model);

gtk_container_add (GTK_CONTAINER (sw), treeview);

  // add columns to the tree view
add_columns(GTK_TREE_VIEW (treeview), FALSE);

gtk_box_pack_start(GTK_BOX(hbox), vbox1, TRUE, TRUE, 0);

if (!title1.empty())
  {
  widget = gtk_label_new(title1.c_str());
  gtk_box_pack_start(GTK_BOX(vbox1), widget, FALSE, TRUE, 0);
  }
gtk_box_pack_start(GTK_BOX(vbox1), sw, TRUE, TRUE, 0);

// If we want another window beside it, put it here in hbox  
// Use this for GTK_SELECTION_MULTIPLE to display the selected names 
// otherwise have an extra entry widget to display the single name
  
if (mode == GTK_SELECTION_MULTIPLE)
  {
  GtkTreeModel *model;
  GtkWidget *vbox2 = gtk_vbox_new (FALSE, 0);
  GtkWidget *hbox2 = gtk_hbox_new (TRUE, 0);
  
  GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);  

  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
             GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
              GTK_POLICY_AUTOMATIC,
              GTK_POLICY_AUTOMATIC);

  if (!title2.empty())
    {
    widget = gtk_label_new(title2.c_str());
    gtk_box_pack_start(GTK_BOX(vbox2), widget, FALSE, TRUE, 0);
    }

  gtk_box_pack_start(GTK_BOX(vbox2), sw, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);

  
  model = GTK_TREE_MODEL(gtk_list_store_new (4, GDK_TYPE_PIXBUF,
                                             G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
//                                           type (F or D)       R or ""    full pathname

  treeview2 = gtk_tree_view_new_with_model(model);
  add_columns(GTK_TREE_VIEW (treeview2), (name_type==reqtype(REQ_DIRECTORIES)) );
  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview2)), GTK_SELECTION_SINGLE);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview2), FALSE);



  g_object_unref (model);
  gtk_container_add (GTK_CONTAINER (sw), treeview2);

  gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, TRUE, 0);
  
  if ( (name_type==REQ_DIRECTORIES) && (!omit_recursive) )
    {
    widget = gtk_button_new_with_label("R-ON / R-OFF");
    decorate_button(GTK_WIDGET(widget), bg_normal,  bg_prelight, NULL);       
    g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_toggle_recursive), (gpointer)this);
    gtk_box_pack_start(GTK_BOX(hbox2), widget, FALSE, TRUE, 0);
    }

  widget = gtk_button_new_with_label("DELETE");
  decorate_button(GTK_WIDGET(widget), bg_normal,  bg_prelight, NULL);
  g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_delete), (gpointer)this);
  gtk_box_pack_start(GTK_BOX(hbox2), widget, FALSE, TRUE, 0);

  }


path_entry = gtk_entry_new();
gtk_entry_set_editable (GTK_ENTRY(path_entry), FALSE);  
gtk_entry_set_text(GTK_ENTRY(path_entry), path.c_str());
  

if (mode == GTK_SELECTION_SINGLE)
  {
  GtkWidget *table =  gtk_table_new(10 + (name_type==reqtype(REQ_DIRECTORIES)) ? 5 : 0,
                                    50,
                                    TRUE);            // height, width, homgeneous
  gtk_box_pack_start(GTK_BOX(vbox1), table, TRUE, TRUE, 0);

  widget = gtk_label_new( "PATH: " );
  gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);
  gtk_label_set_attributes(GTK_LABEL(widget), attr_list);
  gtk_table_attach(GTK_TABLE(table), widget, 0, 10, 0, 5, 
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), 5, 5);

  gtk_table_attach(GTK_TABLE(table), path_entry, 10, 50, 0, 5, 
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), 5, 5);
 

  widget = gtk_label_new( "NAME: " );
  gtk_misc_set_alignment (GTK_MISC (widget), 1, .5);
  gtk_label_set_attributes(GTK_LABEL(widget), attr_list);
  gtk_table_attach(GTK_TABLE(table), widget, 0, 10, 5, 10, 
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), 5, 5);

    
  name_entry = gtk_entry_new();  
  gtk_entry_set_editable (GTK_ENTRY(name_entry), name_editable);  
  gtk_entry_set_text(GTK_ENTRY(name_entry), chosen_name.c_str());
  gtk_table_attach(GTK_TABLE(table), name_entry, 10, 50, 5, 10,
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), 5, 5);

  if (name_type==reqtype(REQ_DIRECTORIES))
      {
      recursive_checkbutton = gtk_check_button_new_with_label("Recursive");
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (recursive_checkbutton), TRUE);
      gtk_table_attach(GTK_TABLE(table), recursive_checkbutton, 0, 50, 10, 15,
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
        (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), 5, 5);
      }
   }
else
  {

  gtk_box_pack_start(GTK_BOX(vbox1), path_entry, FALSE, TRUE, 0);

  widget = gtk_button_new_with_label( (name_type==reqtype(REQ_DIRECTORIES)) ?
                                      "SELECT PATH" : "SELECT FILE" );

  decorate_button(GTK_WIDGET(widget), bg_normal,  bg_prelight, NULL);

  g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_select), (gpointer)this);
  gtk_box_pack_start(GTK_BOX(vbox1), widget, FALSE, TRUE, 0);

  }

  
gtk_widget_show_all(hbox);

return hbox;
}


void Requester::decorate_button(GtkWidget *widget, GdkColor *bg_normal,  GdkColor *bg_prelight, const gchar *stock_id)
{
if (bg_normal) gtk_widget_modify_bg(GTK_WIDGET(widget), GTK_STATE_NORMAL, bg_normal);  
if (bg_prelight) gtk_widget_modify_bg(GTK_WIDGET(widget), GTK_STATE_PRELIGHT, bg_prelight);
if ((stock_id) && *(stock_id))
  {
  gtk_button_set_image(GTK_BUTTON(widget), gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON));
  // Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(widget)))); 
  }
}




void Requester::file_request(string req_title, reqtype req_type, GtkSelectionMode display_type)
{
GtkWidget *widget;
GtkWidget *table;
GdkScreen *screen;
gint sheight;

title = req_title;
  
requester_window = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW(requester_window), title.c_str());

// It may be better to make the size proportional to the height of the screen
screen = gtk_window_get_screen(GTK_WINDOW(requester_window));
sheight = gdk_screen_get_height(screen);
gtk_widget_set_size_request (requester_window, sheight * 2 / 5 * 350 / 400, sheight * 2 / 5);   // width, height

gtk_container_set_border_width(GTK_CONTAINER(requester_window), 0);

g_signal_connect (G_OBJECT (requester_window), "destroy",
		    G_CALLBACK (callback_destroy_requester), (gpointer)NULL);

table = gtk_table_new(400, 350, TRUE);            // height, width
gtk_container_add(GTK_CONTAINER(GTK_DIALOG (requester_window)->vbox), table);

//cout << "Requester::file_request 1" << endl;

widget = file_window(requester_window, req_type, display_type);


//cout << "Requester::file_request 2" << endl;

gtk_table_attach_defaults (GTK_TABLE (table), widget, 0, 350, 0, 400);

widget = gtk_button_new_with_label(cancel_text.c_str());
decorate_button(GTK_WIDGET(widget), bg_normal,  bg_prelight, cancel_stock_id);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_quit_requester), (gpointer)this);
gtk_box_pack_end(GTK_BOX (GTK_DIALOG (requester_window)->action_area), 
                           widget, TRUE, TRUE, 0);

//cout << "Requester::file_request 3" << endl;


widget = gtk_button_new_with_label(ok_text.c_str());
decorate_button(GTK_WIDGET(widget), bg_normal,  bg_prelight, ok_stock_id);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_OK), (gpointer)this);
gtk_box_pack_start(GTK_BOX (GTK_DIALOG (requester_window)->action_area), 
                           widget, TRUE, TRUE, 0);

//cout << "Requester::file_request 4" << endl;


gtk_grab_add(requester_window);
gtk_widget_grab_focus(requester_window);

//cout << "Requester::file_request 5" << endl;


gtk_widget_show_all(requester_window);
  
//cout << "Requester::file_request 6" << endl;


gtk_main ();
}


void Requester::refresh(void)
{
GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(this->treeview));
get_name_lists(path, dir_list, file_list);
fix_list_store(GTK_LIST_STORE(model), this);
}

string Requester::get_request_path(void)
{
return path;
}

// This must be a file list
gboolean Requester::get_request_f_pathname_multiple(list<string> &slist)
{
if (display_mode == GTK_SELECTION_MULTIPLE)
  {
  if (requester_window == NULL)
    {
    gchar *str;
    GtkTreeIter iter;

//  slist = chosename_list;
//cout << "Requester::get_request_name_multiple 1" << endl;

    slist.clear();
//cout << "Requester::get_request_name_multiple 2" << endl;

    
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview2));
//cout << "Requester::get_request_name_multiple 3:-" << endl;

    gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

    while (valid)
      {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                     1, // the column #,
                    &str,
                    -1);

//cout << "        " << str << endl;

      slist.push_back(str);
      g_free(str);
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
      }
    return TRUE;
    }
  else
    {
//    cout << "Requester::get_request_name_multiple - separate requester window" << endl;
    slist = chosen_f_pathname_list;
    return TRUE;
    }
  }
return FALSE;
}

// This must be a directory list and include recursive
gboolean Requester::get_request_d_pathname_multiple(list<Directory> &slist)
{
if (display_mode == GTK_SELECTION_MULTIPLE)
  {
  if (requester_window == NULL)
    {
    gchar *str, *rec;
    GtkTreeIter iter;

    slist.clear();
    
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview2));


    gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

    while (valid)
      {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
                         1, &rec,
                         2, &str,
                         -1);

      Directory d;
      d.name = str;
      d.recursive = (strcmp(rec, "R")==0) ? TRUE : FALSE;
      slist.push_back(d);
      g_free(str);
      g_free(rec);
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
      }
    return TRUE;
    }
  else
    {
//    cout << "Requester::get_request_name_multiple - separate requester window" << endl;
    
    slist = chosen_d_pathname_list;
    return TRUE;
    }
  }
return FALSE;
}

gboolean Requester::get_request_d_pathname_multiple(list<string> &slist)
{
if (display_mode == GTK_SELECTION_MULTIPLE)
  {
  if (requester_window == NULL)
    {
    gchar *str, *rec;
    GtkTreeIter iter;

    slist.clear();
    
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview2));


    gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

    while (valid)
      {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
                         1, &rec,
                         2, &str,
                         -1);

      slist.push_back(str);
      g_free(str);
      g_free(rec);
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
      }
    return TRUE;
    }
  else
    {
    list<Directory>::iterator d;
    
    for (d=chosen_d_pathname_list.begin(); d!=chosen_d_pathname_list.end(); ++d)
      {
      slist.push_back(d->name);
      }
    return TRUE;
    }
  }
return FALSE;
}


// Read direct from treeview2
gboolean Requester::get_request_name_multiple(list<string> &slist)
{
if (display_mode == GTK_SELECTION_MULTIPLE)
  {
  if (requester_window == NULL)
    {
    gchar *str;
    string name;
    GtkTreeIter iter;

//  slist = chosename_list;
//cout << "Requester::get_request_name_multiple:-" << endl;

    slist.clear();
    
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview2));
    gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);

    while (valid)
      {
      gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                    (display_type==reqtype(REQ_FILES)) ? 1 : 2, // the column #,
                    &str,
                    -1);

//cout << "        " << str << endl;
      if (get_name_from_pathname(str, name))
        {
        slist.push_back(name);
        }
      g_free(str);
      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
      }
    return TRUE;
    }
  else
    {
    slist = chosen_name_list;
    return TRUE;
    }
  }
return FALSE;
}

gboolean Requester::get_request_f_pathname_single(string &pathname)
{

if ((display_mode==GTK_SELECTION_SINGLE) && (display_type==reqtype(REQ_FILES)))
  {
  pathname = chosen_f_pathname;
//  cout << "Requester::get_request_pathname_single - \n" << chosen_pathname << "\"" << endl;
  return TRUE;
  }
//cout << "Requester::get_request_pathname_single - FAILED" << endl;
return FALSE;
}

gboolean Requester::get_request_d_pathname_single(Directory &directory)
{

if ((display_mode==GTK_SELECTION_SINGLE) && (display_type==reqtype(REQ_DIRECTORIES)))
  {
  directory = chosen_d_pathname;
//  cout << "Requester::get_request_pathname_single - \n" << chosen_pathname << "\"" << endl;
  return TRUE;
  }
//cout << "Requester::get_request_pathname_single - FAILED" << endl;
return FALSE;
}



gboolean Requester::get_request_name_single(string &name)
{
if (display_mode == GTK_SELECTION_SINGLE)
  {
  name = chosen_name;
  return TRUE;
  }
return FALSE;
}

GtkSelectionMode Requester::get_mode(void)
{
return display_mode;
}

reqtype Requester::get_type(void)
{
return display_type;
}

gboolean Requester::requester(string req_title, reqtype req_type, GtkSelectionMode display_type)
{
//cout << "Requester::requester 1" << endl;

file_request(req_title, req_type, display_type);

//cout << "Requester::requester 2" << endl;

return reply_flag;
}







