/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       requester.h                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       11-Apr-2010 11.26.35 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Header file                                                               *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __FILE_REQUESTER_H__
#define __FILE_REQUESTER_H__

#include <iostream>
#include <list>
#include <string>

#include <gtk/gtk.h>

using namespace std;

enum reqtype{ REQ_FILES, REQ_DIRECTORIES };

class Directory
{
public:
  string name;
  bool recursive;
};

class Requester
{
public:
  Requester(void);
  ~Requester(void);
  
  GtkWidget *file_window(GtkWidget *parent, reqtype name_type, GtkSelectionMode mode);
  int requester(string req_title, reqtype req_type, GtkSelectionMode display_type);

  void set_name_editable(gboolean editable);
  void set_recursive_not_included(void);

  void set_title1(string text);
  void set_title2(string text);

  void set_ok_text(string text);
  void set_cancel_text(string text);

  void set_decorate_buttons(GdkColor *bg_normal,  GdkColor *bg_prelight,
                        const gchar *cancel_stock_id, const gchar *ok_stock_id);

  void refresh(void);
  void clear_window2(Requester *fr);
  void append_treeview2(gchar *str, bool recursive, Requester *fr);

  string get_request_path(void);
  
  gboolean get_request_f_pathname_single(string &pathname);
  gboolean get_request_d_pathname_single(Directory &directory);

  gboolean get_request_name_single(string &name); 
  gboolean get_request_name_multiple(list<string> &slist);

  gboolean get_request_f_pathname_multiple(list<string> &slist);
  gboolean get_request_d_pathname_multiple(list<Directory> &slist);
  gboolean get_request_d_pathname_multiple(list<string> &slist);

  GtkSelectionMode get_mode(void);
  reqtype get_type(void);

private:
  static void callback_destroy_requester(GtkWidget *widget, gpointer data);
  static void callback_quit_requester(GtkWidget *widget,  Requester *fr);
  static void callback_OK(GtkWidget *widget, Requester *fr);
  static void callback_select(GtkWidget *widget, Requester *fr);
  static void callback_toggle_recursive(GtkWidget *widget, Requester *fr);
  static void callback_delete(GtkWidget *widget, Requester *fr);

  static void callback_list_select(GtkWidget *widget, Requester *fr);
  static void callback_double_click(GtkTreeView *view,
                                      GtkTreePath *path,
                                      GtkTreeViewColumn *column,
                                      Requester *fr);

  gboolean isfile(const char *pathname);
  gboolean isdir(const char *pathname);
  bool get_name_from_pathname(gchar *pathname, string &name);
  gboolean get_name_lists(string directory, list<string> &dirlist, list<string> &filelist);

  void fix_list_store(GtkListStore *store, Requester *fr);
  GtkTreeModel *create_model(void);
  void add_columns (GtkTreeView *treeview, bool extra);
  void file_request(string req_title, reqtype req_type, GtkSelectionMode display_type);
  bool in_list(list<string>, string s);

  void decorate_button(GtkWidget *widget, GdkColor *bg_normal,  GdkColor *bg_prelight, const gchar *stock_id);

  string title;
  string title1;
  string title2;

  GdkPixbuf *directory_pixbuf;
  GdkPixbuf *file_pixbuf;
  GdkPixbuf *exec_pixbuf;

  GtkSelectionMode display_mode;
  reqtype display_type;

  list<string> dir_list;
  list<string> file_list;
  
  list<string> chosen_f_pathname_list;
  list<Directory> chosen_d_pathname_list;
  list<string> chosen_name_list;
  
  string chosen_f_pathname;  
  Directory chosen_d_pathname;  

  string chosen_name;

  GtkWidget *requester_window;
  GtkWidget *treeview;
  GtkWidget *treeview2;
  string path;

  GtkWidget *path_entry;
  GtkWidget *name_entry;
  GtkWidget *recursive_checkbutton;
  
  PangoAttribute *attr;
  PangoAttrList *attr_list;

  string ok_text;
  string cancel_text;
  
  static GdkColor *bg_normal;
  static GdkColor *bg_prelight;
  static const gchar *cancel_stock_id;
  static const gchar *ok_stock_id;
  
  gboolean name_editable;

  gboolean omit_recursive;

  gboolean reply_flag;

protected:


};

#endif // __FILE_REQUESTER_H__
